import logging
import traceback
from django.core.exceptions import ImproperlyConfigured

from .utils import Utils
from django.conf import settings
from log_labeler import local, LOG_LABEL_REQUEST_SETTING, DEFAULT_HEADER_VALUE, \
        MAX_REQUEST_RESPONSE_SIZE, DEFAULT_LOG_LEVEL, \
        NIM_DJANGO_REQUEST_LOG_LEVEL_NAME, LOGGING

logger = logging.getLogger(__name__)


class HeaderToLabelMiddleware(object):
    """
    Provides full logging of requests and responses
    """
    _initial_http_body = b""
    _request_start_time = None
    _TRUNCATE_INDICATOR = "---[TRUNCATED]---"
    _OBFUSCATE_INDICATOR = "---[HIDDEN]---"
    _REQUEST = "Request"
    _RESPONSE = "Response"
    _ERROR = "Error"

    def __init__(self, get_response=None):
        if not hasattr(settings, LOG_LABEL_REQUEST_SETTING):
            raise ImproperlyConfigured(f"Please set {LOG_LABEL_REQUEST_SETTING}, it is the dictionary of header names and values that need to be appended to the log")

        if not hasattr(settings, MAX_REQUEST_RESPONSE_SIZE):
            raise ImproperlyConfigured(f"Please set {MAX_REQUEST_RESPONSE_SIZE}, it is the value of the max length of the response")

        if not hasattr(settings, DEFAULT_LOG_LEVEL):
            raise ImproperlyConfigured(f"Please set {DEFAULT_LOG_LEVEL}, it is the value of the log level")

        if not hasattr(settings, NIM_DJANGO_REQUEST_LOG_LEVEL_NAME):
            raise ImproperlyConfigured(f"Please set {NIM_DJANGO_REQUEST_LOG_LEVEL_NAME}, it is the value of the header name containing the correlation id")

        self.get_response = get_response

    def process_request(self, request):
        self._initial_http_body = request.body
        self._request_start_time = Utils.get_time_in_milliseconds()
        log_label_request_settings = getattr(settings, LOG_LABEL_REQUEST_SETTING, dict())

        headers = Utils.obfuscate_headers(Utils.get_all_headers(request))

        log_entry_info = dict(
            type=self._REQUEST,
            request_start_time_ms=self._request_start_time,
            method=request.method,
            url=Utils.obfuscate_url(request.build_absolute_uri())
        )
        log_entry_info.update(headers)

        log_entry_debug = dict(
            type=self._REQUEST,
            request_start_time_ms=self._request_start_time,
            request_end_time_ms=Utils.get_time_in_milliseconds(),
            method=request.method,
            body=Utils.obfuscate_body(Utils.adjust_string_length(self._initial_http_body, getattr(settings, MAX_REQUEST_RESPONSE_SIZE))),
            url=Utils.obfuscate_url(request.build_absolute_uri())
        )
        log_entry_debug.update(headers)

        for label, header_name in log_label_request_settings.items():
            header_value = Utils.get_header_by_name(request, header_name, DEFAULT_HEADER_VALUE)
            setattr(local, label, header_value)
            setattr(request, label, header_value)
            log_entry_info[label] = header_value
            log_entry_debug[label] = header_value

        logger.debug('', extra=log_entry_debug)
        logger.info('', extra=log_entry_info)

    def process_response(self, request, response):
        headers = Utils.obfuscate_headers(Utils.get_all_headers(request))
        log_entry = dict(
            type=self._RESPONSE,
            request_start_time_ms=self._request_start_time,
            request_end_time_ms=Utils.get_time_in_milliseconds(),
            method=request.method,
            body=Utils.obfuscate_body(Utils.adjust_string_length(self._initial_http_body,
                                            getattr(settings, MAX_REQUEST_RESPONSE_SIZE)) if response.status_code < 400 else Utils.obfuscate_body(self._initial_http_body.decode("UTF-8"))),
            status_code=response.status_code,
            response=Utils.obfuscate_response(Utils.adjust_string_length(response.content,
                                                settings.MAX_REQUEST_RESPONSE_SIZE) if response.status_code < 400 else Utils.obfuscate_body(response.content.decode("UTF-8"))),
            url=Utils.obfuscate_url(request.build_absolute_uri())
        )
        log_entry.update(headers)

        log_entry_info = dict(
            type=self._RESPONSE,
            request_start_time_ms=self._request_start_time,
            request_end_time_ms=Utils.get_time_in_milliseconds(),
            status_code=response.status_code,
            method=request.method,
            url=Utils.obfuscate_url(request.build_absolute_uri())
        )
        log_entry_info.update(log_entry_info)
        log_entry_info.update(headers)

        log_label_request_settings = getattr(settings, LOG_LABEL_REQUEST_SETTING, dict())
        for label in log_label_request_settings:
            log_entry[label] = getattr(request, label)
            log_entry_info[label] = getattr(request, label)

        if response.status_code < 400:
            logger.debug(
                '', extra=log_entry
            )
        else:
            logger.error(
                '', extra=log_entry
            )

        logger.info(
            '', extra=log_entry_info
        )
        return response

    def process_exception(self, request, exception):
        headers = Utils.obfuscate_headers(Utils.get_all_headers(request))
        log_entry = dict(
            type=self._ERROR,
            request_start_time_ms=self._request_start_time,
            request_end_time_ms=Utils.get_time_in_milliseconds(),
            method=request.method,
            body=Utils.obfuscate_body(Utils.adjust_string_length(self._initial_http_body, getattr(settings, MAX_REQUEST_RESPONSE_SIZE))),
            status_code=500,
            exception=str(exception),
            url=Utils.obfuscate_url(request.build_absolute_uri()),
            stack_trace=traceback.format_exc()
        )
        log_entry.update(headers)

        log_entry_info = dict(
            type=self._ERROR,
            request_start_time_ms=self._request_start_time,
            request_end_time_ms=Utils.get_time_in_milliseconds(),
            status_code=500,
            method=request.method,
            url=Utils.obfuscate_url(request.build_absolute_uri())
        )
        log_entry_info.update(headers)

        log_label_request_settings = getattr(settings, LOG_LABEL_REQUEST_SETTING, dict())
        for label in log_label_request_settings:
            log_entry[label] = getattr(request, label)
            log_entry_info[label] = getattr(request, label)

        logger.error(
            '', extra=log_entry
        )
        logger.info(
            '', extra=log_entry_info
        )
        return exception



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

from setuptools import setup
import re
import os
import sys
import codecs

name = 'django-log-labeler'
package = 'log_labeler'
description = 'Django middleware and log filter to attach a unique ID to every log message generated as part of a request'
url = 'https://psbl102zatcrh.vodacom.corp/nimble-backend/Vodacom-Django-Log-Labeler/'
author = 'Hermann'
author_email = 'hermann.ntsamo@vcontractor.co.za'
license = 'BSD'
install_requires = ["django>=1.9.2", "requests>=2.22.0", "twine==3.1.1"]

with codecs.open('README.md', encoding='utf-8') as f:
    readme = f.read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s'" % args)
    print("  git push --tags")
    sys.exit()


setup(
    name=name,
    version='1.0.6',
    url=url,
    license=license,
    description=description,
    long_description=readme,
    long_description_content_type='text/markdown',
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=install_requires
)



import re
import math
import time
from django.conf import settings
from log_labeler import LOG_LABEL_OBFUSCATE

class Utils:
    @classmethod
    def array_difference(cls, array1, array2):
        return list(set(array1) - set(array2))

    @classmethod
    def array_intersection(cls, array1, array2):
        return list(set(array1) & set(array2))

    @classmethod
    def remove_new_lines(cls, data):
        return data.replace("\n", "").replace("\r", "")

    @classmethod
    def get_nim_headers(cls, request):
        NIM_HEADER_REG_EXP = r"^HTTP\_NIM\_"
        HTTP_REG_EXP = r"^HTTP\_"
        nim_headers = dict()
        for key, value in request.META.items():
            if re.match(NIM_HEADER_REG_EXP, key):
                transformed_key = re.sub(HTTP_REG_EXP, "", key).replace("_", "-")
                nim_headers[transformed_key.lower()] = value

        return nim_headers

    @classmethod
    def get_all_headers(cls, request):
        headers = dict()
        for key, value in request.META.items():
            headers[key.lower()] = value

        return headers

    @classmethod
    def get_header_by_name(cls, request, header_name, default_value):
        return request.META.get(header_name.upper(), default_value)

    @classmethod
    def adjust_string_length(cls, value, max_length):
        TRUNCATE_INDICATOR = "---[TRUNCATED]---"
        if isinstance(value, bytes):
            value = value.decode("UTF-8")
        output = value

        if max_length and max_length.upper() != "OFF":
            max_length = int(max_length)
            value = value
            length = len(value)
            if length > max_length:
                extra_chars = int(math.floor((length - max_length) / 2))
                adjust_even = 1 if length % 2 == 0 else 0
                middle = int(math.ceil(length / 2))
                output = "".join([value[:middle - extra_chars], TRUNCATE_INDICATOR, value[middle + extra_chars + adjust_even:]])
        return output

    @classmethod
    def get_time_in_milliseconds(cls):
        return int(round(time.time() * 1000))

    @classmethod
    def obfuscate_headers(cls, headers):
        _headers = headers.copy()
        log_label_obfuscate = getattr(settings, LOG_LABEL_OBFUSCATE, dict())
        if "headers" in log_label_obfuscate:
            headers_to_obfuscate = cls.array_intersection(log_label_obfuscate["headers"], _headers)
            for header_name in headers_to_obfuscate:
                _headers[header_name] = "[--HIDDEN--]"

        return _headers

    @classmethod
    def obfuscate_body(cls, body):
        body = cls.remove_new_lines(body)
        log_label_obfuscate = getattr(settings, LOG_LABEL_OBFUSCATE, dict())
        if "body" in log_label_obfuscate:
            for pattern, replacement in log_label_obfuscate["body"].items():
                if re.search(pattern, body):
                    body = re.sub(pattern, replacement, body)

        return body

    @classmethod
    def obfuscate_response(cls, response):
        response = cls.remove_new_lines(response)
        log_label_obfuscate = getattr(settings, LOG_LABEL_OBFUSCATE, dict())
        if "response" in log_label_obfuscate:
            for pattern, replacement in log_label_obfuscate["response"].items():
                if re.search(pattern, response):
                    response = re.sub(pattern, replacement, response)

        return response

    @classmethod
    def obfuscate_url(cls, url):
        log_label_obfuscate = getattr(settings, LOG_LABEL_OBFUSCATE, dict())
        if "url" in log_label_obfuscate:
            for pattern, replacement in log_label_obfuscate["url"].items():
                if re.search(pattern, url):
                    url = re.sub(pattern, replacement, url)

        return url
